export type AseColorType = 'global' | 'spot' | 'normal';
export interface AseColor {
    name: string;
    color: {
        r: number;
        g: number;
        b: number;
        type: AseColorType;
    } | {
        c: number;
        m: number;
        y: number;
        k: number;
        type: AseColorType;
    } | {
        k: number;
        type: AseColorType;
    } | {
        l: number;
        a: number;
        b: number;
        type: AseColorType;
    };
}
export interface AseGroup {
    name: string;
    colors: AseColor[];
}
export interface Ase {
    colors: (AseGroup | AseColor)[];
}
export declare function readAse(buffer: ArrayBufferView): Ase;
